@@ -1,7 +1,11 @@
 #!/usr/bin/env bash
 
+set -o pipefail
+shopt -s failglob
+set -u 
+
 # Allow us to debug what's happening in the script if necessary
-if [ "$STEAM_DEBUG" ]; then
+if [ "${STEAM_DEBUG-}" ]; then
 	set -x
 fi
 export TEXTDOMAIN=steam
@@ -15,9 +19,13 @@ ARCHIVE_EXT=tar.xz
 # directory lives - and all this in a subshell, so we don't affect
 # $PWD
 
-STEAMROOT="$(cd "${0%/*}" && echo $PWD)"
+STEAMROOT="$(cd $(dirname $0) && echo $PWD)"
+if [ -z ${STEAMROOT} ]; then
+	echo $"Couldn't find Steam root directory from "$0", aborting!"
+	exit 1
+fi
 STEAMDATA="$STEAMROOT"
-if [ -z $STEAMEXE ]; then
+if [ -z ${STEAMEXE-} ]; then
   STEAMEXE=`basename "$0" .sh`
 fi
 # Backward compatibility for server operators
@@ -31,12 +39,9 @@ if [ "$STEAMEXE" = "steamcmd" ]; then
 fi
 cd "$STEAMROOT"
 
-# The minimum version of the /usr/bin/steam script that we require
-MINIMUM_STEAMSCRIPT_VERSION=100020
-
 # Save the system paths in case we need to restore them
 export SYSTEM_PATH="$PATH"
-export SYSTEM_LD_LIBRARY_PATH="$LD_LIBRARY_PATH"
+export SYSTEM_LD_LIBRARY_PATH="${LD_LIBRARY_PATH-}"
 
 function show_message()
 {
@@ -121,10 +126,6 @@ function show_license_agreement()
 	fi
 }
 
-function warn_outofdate()
-{
-	show_message --warning $"Your steam package is out of date. Please get an updated version from your package provider or directly from http://repo.steampowered.com/steam for supported distributions."
-}
 
 function distro_description()
 {
@@ -212,30 +213,6 @@ function detect_package()
 	echo "$STEAMPACKAGE"
 }
 
-function detect_scriptversion()
-{
-	SCRIPT_VERSION=$(fgrep "$2=" "$1")
-	if [[ "$SCRIPT_VERSION" ]]; then
-		expr "$SCRIPT_VERSION" : ".*=\(.*\)"
-	else
-		echo "0"
-	fi
-}
-
-# Check a currently installed script against a new script and see if the
-# installed one needs to be updated.
-function check_scriptversion()
-{
-	SCRIPT=$1
-	VERSION_TOKEN=$2
-	MINIMUM_VERSION=$3
-
-	VERSION="$(detect_scriptversion "$SCRIPT" $VERSION_TOKEN)"
-	if [[ "$VERSION" -lt "$MINIMUM_VERSION" ]]; then
-		return 1
-	fi
-	return 0
-}
 
 function detect_steamdatalink()
 {
@@ -336,11 +313,9 @@ function extract_archive()
 function has_runtime_archive()
 {
 	# Make sure we have files to unpack
-	for file in "$STEAM_RUNTIME.$ARCHIVE_EXT".part*; do
-		if [ ! -f "$file" ]; then
-			return 1
-		fi
-	done
+    if [ ! -f "$STEAM_RUNTIME.$ARCHIVE_EXT.part0" ]; then
+		return 1
+	fi
 
 	if [ ! -f "$STEAM_RUNTIME.checksum" ]; then
 		return 1
@@ -403,7 +378,7 @@ function get_missing_libraries()
 		# We couldn't run the link loader for this architecture
 		echo "libc.so.6"
 	else
-		LD_PRELOAD= ldd "$1" | grep "=>" | grep -v linux-gate | grep -v / | awk '{print $1}'
+		LD_PRELOAD= ldd "$1" | grep "=>" | grep -v linux-gate | grep -v / | awk '{print $1}' || true
 	fi
 }
 
@@ -426,6 +401,12 @@ function ignore_signal()
 
 function reset_steam()
 {
+	# Ensure STEAMROOT is defined to something reasonable so we don't wipe the wrong thing
+	if [ -z "${STEAMROOT}" ]; then
+		show_message --error $"Couldn't find Steam directory, it's not safe to reset Steam. Please contact technical support."
+		return 1
+	fi
+
 	# Don't wipe development files
 	if [ -f "$STEAMROOT/steam_dev.cfg" ]; then
 		echo "Can't reset development directory"
@@ -442,6 +423,11 @@ function reset_steam()
 		return 1
 	fi
 
+	if [ "$STEAMROOT" = "" ]; then
+		show_message --error $"Couldn't find Steam, it's not safe to reset Steam. Please contact technical support."
+		return 1
+	fi
+
 	STEAM_SAVE="$STEAMROOT/.save"
 
 	# Don't let the user interrupt us, or they may corrupt the install
@@ -455,7 +441,7 @@ function reset_steam()
 	# Back up games and critical files
 	# Backup package dir so that we're not hitting CDNs if there is no manifest change
 	mkdir -p "$STEAM_SAVE"
-	for i in bootstrap.tar.xz ssfn* SteamApps userdata package; do
+	for i in bootstrap.tar.xz ssfn* SteamApps steamapps userdata package; do
 		if [ -e "$i" ]; then
 			mv -f "$i" "$STEAM_SAVE/"
 		fi
@@ -464,8 +450,10 @@ function reset_steam()
 		mv -f "$i" "$i.bak"
 	done
 
-	# Scary!
-	rm -rf "$STEAMROOT/"*
+	# Check before removing
+	if [ "$STEAMROOT" != "" ]; then
+		rm -rf "$STEAMROOT/"*
+	fi
 
 	# Move things back into place
 	mv -f "$STEAM_SAVE/"* "$STEAMROOT/"
@@ -511,8 +499,8 @@ fi
 
 # identify Linux distribution and pick an optimal bin dir
 PLATFORM=`detect_platform`
-PLATFORM32=`echo $PLATFORM | fgrep 32`
-PLATFORM64=`echo $PLATFORM | fgrep 64`
+PLATFORM32=`echo $PLATFORM | grep 32 || true`
+PLATFORM64=`echo $PLATFORM | grep 64 || true`
 if [ -z "$PLATFORM32" ]; then
 	PLATFORM32=`echo $PLATFORM | sed 's/64/32/'`
 fi
@@ -537,6 +525,7 @@ STEAMDATALINK="`detect_steamdatalink`" # points at the Steam content path
 STEAMSTARTING="$STEAMCONFIG/starting"
 
 # Was -steamos specified
+: "${STEAMOS:=}"
 if steamos_arg $@; then
 	STEAMOS=1
 fi
@@ -544,9 +533,11 @@ fi
 # See if this is the initial launch of Steam
 if [ ! -f "$PIDFILE" ] || ! kill -0 $(cat "$PIDFILE") 2>/dev/null; then
 	INITIAL_LAUNCH=true
+else
+	INITIAL_LAUNCH=false
 fi
 
-if [ "$1" = "--reset" ]; then
+if [ "${1-}" = "--reset" ]; then
 	reset_steam
 	exit
 fi
@@ -555,16 +546,9 @@ if [ "$INITIAL_LAUNCH" ]; then
 	# Show the license agreement, if needed
 	show_license_agreement
 
-	# See if we need to update the /usr/bin/steam script
-	if [ -z "$STEAMSCRIPT" ]; then
+	if [ -z "${STEAMSCRIPT:-}" ]; then
 		STEAMSCRIPT="/usr/bin/`detect_package`"
 	fi
-	if [ -f "$STEAMSCRIPT" ]; then
-		if ! check_scriptversion "$STEAMSCRIPT" STEAMSCRIPT_VERSION "$MINIMUM_STEAMSCRIPT_VERSION"; then
-			STEAMSCRIPT_OUTOFDATE=1
-			warn_outofdate
-		fi
-	fi
 
 	# Install any additional dependencies
 	if [ -z "$STEAMOS" ]; then
@@ -611,15 +595,16 @@ echo "Running Steam on $(distro_description)"
 # as possible, so feel free to tinker with it and submit patches and
 # bug reports.
 #
+: "${STEAM_RUNTIME:=}"
 if [ "$STEAM_RUNTIME" = "debug" ]; then
 	# Use the debug runtime if it's available, and the default if not.
 	export STEAM_RUNTIME="$STEAMROOT/$PLATFORM/steam-runtime"
 
 	if unpack_runtime; then
-		if [ -z "$STEAM_RUNTIME_DEBUG" ]; then
+		if [ -z "${STEAM_RUNTIME_DEBUG-}" ]; then
 			STEAM_RUNTIME_DEBUG="$(cat "$STEAM_RUNTIME/version.txt" | sed 's,-release,-debug,')"
 		fi
-		if [ -z "$STEAM_RUNTIME_DEBUG_DIR" ]; then
+		if [ -z "{$STEAM_RUNTIME_DEBUG_DIR-}" ]; then
 			STEAM_RUNTIME_DEBUG_DIR="$STEAMROOT/$PLATFORM"
 		fi
 		if [ ! -d "$STEAM_RUNTIME_DEBUG_DIR/$STEAM_RUNTIME_DEBUG" ]; then
@@ -674,18 +659,15 @@ if [ "$STEAM_RUNTIME" -a "$STEAM_RUNTIME" != "0" ]; then
 				;;
 		esac
 
-		export LD_LIBRARY_PATH="$STEAM_RUNTIME/i386/lib/i386-linux-gnu:$STEAM_RUNTIME/i386/lib:$STEAM_RUNTIME/i386/usr/lib/i386-linux-gnu:$STEAM_RUNTIME/i386/usr/lib:$STEAM_RUNTIME/amd64/lib/x86_64-linux-gnu:$STEAM_RUNTIME/amd64/lib:$STEAM_RUNTIME/amd64/usr/lib/x86_64-linux-gnu:$STEAM_RUNTIME/amd64/usr/lib:$LD_LIBRARY_PATH"
+		export LD_LIBRARY_PATH="$STEAM_RUNTIME/i386/lib/i386-linux-gnu:$STEAM_RUNTIME/i386/lib:$STEAM_RUNTIME/i386/usr/lib/i386-linux-gnu:$STEAM_RUNTIME/i386/usr/lib:$STEAM_RUNTIME/amd64/lib/x86_64-linux-gnu:$STEAM_RUNTIME/amd64/lib:$STEAM_RUNTIME/amd64/usr/lib/x86_64-linux-gnu:$STEAM_RUNTIME/amd64/usr/lib:${LD_LIBRARY_PATH-}"
 	else
 		echo "Unpack runtime failed, error code $?"
 		show_message --error $"Couldn't set up the Steam Runtime. Are you running low on disk space?\nContinuing..."
 	fi
 fi
 
-# Need to add /usr/lib32 to the library path to pick up libvdpau_nvidia.so on Ubuntu 12.04
-export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:/usr/lib32"
-
 # prepend our lib path to LD_LIBRARY_PATH
-export LD_LIBRARY_PATH="$STEAMROOT/$PLATFORM:$LD_LIBRARY_PATH"
+export LD_LIBRARY_PATH="$STEAMROOT/$PLATFORM:$STEAMROOT/$PLATFORM/panorama:${LD_LIBRARY_PATH-}"
 
 # Check to make sure the user will be able to run steam...
 if [ -z "$STEAMOS" ]; then
@@ -694,7 +676,6 @@ fi
 
 # disable SDL1.2 DGA mouse because we can't easily support it in the overlay
 export SDL_VIDEO_X11_DGAMOUSE=0
-ulimit -n 2048 2>/dev/null
 
 # Touch our startup file so we can detect bootstrap launch failure
 if [ "$UNAME" = "Linux" ]; then
@@ -705,12 +686,13 @@ MAGIC_RESTART_EXITCODE=42
 SEGV_EXITCODE=139
 
 # and launch steam
-STEAM_DEBUGGER=$DEBUGGER
+STEAM_DEBUGGER=${DEBUGGER-}
 unset DEBUGGER # Don't use debugger if Steam launches itself recursively
 if [ "$STEAM_DEBUGGER" == "gdb" ] || [ "$STEAM_DEBUGGER" == "cgdb" ]; then
 	ARGSFILE=$(mktemp $USER.steam.gdb.XXXX)
 
 	# Set the LD_PRELOAD varname in the debugger, and unset the global version. 
+	: "${LD_PRELOAD=}"
 	if [ "$LD_PRELOAD" ]; then
 		echo set env LD_PRELOAD=$LD_PRELOAD >> "$ARGSFILE"
 		echo show env LD_PRELOAD >> "$ARGSFILE"
@@ -720,6 +702,7 @@ if [ "$STEAM_DEBUGGER" == "gdb" ] || [ "$STEAM_DEBUGGER" == "cgdb" ]; then
 	$STEAM_DEBUGGER -x "$ARGSFILE" --args "$STEAMROOT/$STEAMEXEPATH" "$@"
 	rm "$ARGSFILE"
 elif [ "$STEAM_DEBUGGER" == "valgrind" ]; then
+    : "${STEAM_VALGRIND:=}"
 	DONT_BREAK_ON_ASSERT=1 G_SLICE=always-malloc G_DEBUG=gc-friendly valgrind --error-limit=no --undef-value-errors=no --suppressions=$PLATFORM/steam.supp $STEAM_VALGRIND "$STEAMROOT/$STEAMEXEPATH" "$@" 2>&1 | tee steam_valgrind.txt
 elif [ "$STEAM_DEBUGGER" == "callgrind" ]; then
     valgrind --tool=callgrind --instr-atstart=no "$STEAMROOT/$STEAMEXEPATH" "$@"
@@ -738,8 +721,7 @@ if [ "$UNAME" = "Linux" ]; then
 	if [ "$INITIAL_LAUNCH" -a \
 	     $STATUS -ne $MAGIC_RESTART_EXITCODE -a \
 	     -f "$STEAMSTARTING" -a \
-	     -z "$STEAM_INSTALLED_BOOTSTRAP" -a \
-	     -z "$STEAMSCRIPT_OUTOFDATE" ]; then
+	     -z "${STEAM_INSTALLED_BOOTSTRAP-}" ]; then
 		# Launching the bootstrap failed, try reinstalling
 		if reset_steam; then
 			# We were able to reinstall the bootstrap, try again
